---
layout: post
title:  "安卓简易游戏开发"
date:   2016-05-04
categories: 游戏设计
excerpt: 安卓
---
* content
{:toc}

## 简介

原本准备做一个弹幕游戏的，由于时间不足只做好了框架。

具体代码以后会发上来。

包含以下几个部分

* 360度摇杆
* 简单的弹幕生成
* 死亡和擦弹判定
* 读取脚本以生成剧本

虽然是很粗糙的设计但是基本框架已经比较完善，下面来详细介绍在安卓这类非专门游戏语言的平台上编写游戏的方法。

---

## 框架的建立

### 游戏核心的循环

对于游戏来说，最重要的就是实现一个"循环"，这样才能不停的绘制游戏画面。然而在循环过程中还得处理玩家的输入，这对c语言是比较困难的。所幸在安卓里提供了一个多线程函数，因此这个最大的问题就解决了。

在循环里除了要做循环之外，也得处理游戏元素之间的逻辑。因此我们写出了一个抽象的框架。

	循环(锁定每1/60s执行一次){
		逻辑运算()
		绘制()
	}
	
每次在游戏里添加其他的元素，都可能需要为其编写逻辑和绘制函数，然后在这个循环里调用它。

这个框架有其自身的缺陷，比如如果逻辑运行时间过长，整个游戏就会卡顿。不过更多时候是绘制时间开销更大，由于对安卓特性尚不了解，我选用了非常基础的canvas类，然而卡顿时有发生，尚未解决。

上面的抽象代码，放在安卓里就是这样

	final Handler myHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			if (msg.what == 0x1233) {
				logic();		//处理逻辑
				invalidate();	//绘制图像
			}
		}
	};

	new Timer().schedule(new TimerTask() {
		@Override
		public void run() {
			// 发送空消息
			myHandler.sendEmptyMessage(0x1233);
		}
	}, 0, getDrawInterval());

这方法不一定是最好的，但是可用。

### 数据的交互

游戏元素之间的交互错综复杂，采用类之间互传参数的做法会导致代码相当繁琐。将所有类都建立在一个核心类下，然后将核心类的引用传递至各个类中是我目前想到的比较好的处理方法，这个核心类也能很方便的执行上面的循环，因此我将他们整合为一个类。

仔细想想，安卓很多代码都会传一个this参数，应该是一样的道理。

### 数据的规范

为了适配不同的分辨率和帧数以及更多可能情况，将数据写为相对形式是很有必要的。比如“坐标为(200,200)就没有坐标为(10%,20%)泛用性广。不过事实上这样的工作量是极大的，我的代码并没有做到这一点。如果只是想练习一下，按自己手机的分辨率来编写代码就可以了。

### 向下细分

在核心类中完成循环部分和各个类的实例化以及核心数据(分辨率等核心数据与分数等游戏中常用数据)之后，其他部分的代码应该尽量移至其他类中以减小核心类的负担。比如敌人的处理，建立一个Enemy类之后，再建立一个Enemys类对其管理可以让核心类变得简洁。

---

## 具体部分的实现

### 脚本与读取

以下部分不具备普适性，为自己在写躲避弹幕游戏中的一点经验。

弹幕游戏的敌人极多而且经常重复，但是如果用代码描述他们，可读性，维护性都不好。因此我编写了一个脚本读取类来读取一个“剧本”，以下截取剧本的一部分

	# level1
	Wait 50
	Enemy3 1 50 20
	Wait 50
	Enemy3 1 50 20
	Wait 50
	Enemy3 1 50 20
	Wait 50
	Enemy3 1 50 20
	
这样写不仅能节约代码，也省略了很多繁琐的设计。比如Wait函数在框架中并不好写，想了很多方法后我选用了脚本，代码量大大降低了。

最重要的是，采用相对时间的描述，修改弹幕（比如插入新弹幕）变得极为容易。

### 360度摇杆

摇杆类复制粘贴自网上，自己也尝试做了点修改，摇杆类额外的采用安卓中的onTouchEvent。手感不是很好。

计算角度之类的比较繁琐，建议了解atan2函数。

### 弹幕生成

非常繁琐的部分，充斥着大量的角度运算，目前只能生成扇形弹幕。

---

### 参考

[github-枪林弹雨](https://github.com/beautifulzzzz/Android/tree/master/枪林弹雨)

